/*
 *  pmutil.c
 *
 *  Manages net interfaces. Sets and clears promiscuous mode
 *  May, 1.996  by  SefuX   <sefux@golum.etsiig.uniovi.es>
 *
 *  Not rights reserved. Use this source freely, but do not say it is yours
 */

/****************/
/* Header files */
/****************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <net/if.h>
#include <time.h>
#include <fcntl.h>
#include <sys/stat.h>

/*************/
/* Constants */
/*************/

#define MAX_IFS 8

/****************/
/* Main program */
/****************/

main (int argc, char * argv[])  {

    static char List[MAX_IFS][16];
    static unsigned int Num;
    static char AuxBuff[4096];
    static unsigned int NumIfs, i, j;
    static struct ifconf ifc;
    static struct ifreq ifr;
    static unsigned int Clear;
    int sd;
    char * p;

    printf ("\nPromiscuous Interfaces Management  [SefuX - 1.996]\n\n");

    if (argc == 1)  {

    /************************************************/
    /* Get info about net interfaces in this system */
    /************************************************/

      if ((sd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)  {
        perror ("socket");
        exit (-1);
      }

      ifc.ifc_len = sizeof (AuxBuff);
      ifc.ifc_buf = AuxBuff;
      if (ioctl (sd, SIOCGIFCONF, (char *) &ifc) < 0)  {
        perror ("ioctl(SIOCGIFCONF)");
        exit (-1);
      }

    /*******************************/
    /* We take a look at that data */
    /*******************************/
 
      NumIfs = (unsigned int) ifc.ifc_len / sizeof (struct ifreq);

      p = (char *) ifc.ifc_req;  /* Points to the first interface */
      Num = 0;

      for (i=0;i<NumIfs;i++)  {
        strcpy (ifr.ifr_name, ((struct ifreq *) p)->ifr_name);
        j=0;
        while ((j<Num) && (strcmp (ifr.ifr_name, List[j])))  {
	  j++;
	}

	if (j>=Num)  {
  	  strcpy (List[Num], ifr.ifr_name);
	  Num++;

          if (ioctl (sd, SIOCGIFFLAGS, &ifr) == -1)  {
            perror ("ioctl(SIOCGIFFLAGS)");
            exit (-1);
          }

          printf ("Interface name :   %s", ifr.ifr_name);

          if (ifr.ifr_flags & IFF_LOOPBACK)  {
            printf ("    [loopback device]");
          }

          printf ("\n");

          if (ifr.ifr_flags & IFF_PROMISC)  {
            printf ("  * PROMISCUOUS MODE ENABLED *\n");
          }
 
          printf ("\n");
	}

        p = p + sizeof (struct ifreq);  /* Get next interface */
      }

      close (sd);
    }

    else

    if (argc == 2)  {

      /**************************************************/
      /* There is an interface name on the command line */
      /**************************************************/

      if (argv[1][0] == '-')  {
        Clear = 1;
        strcpy (AuxBuff, &(argv[1][1]));
      }
      else  {
        Clear = 0;
        strcpy (AuxBuff, argv[1]);
      }

      /****************************************************/
      /* Clear or set promiscuous mode for that interface */
      /****************************************************/

      if ((sd = socket (AF_INET, SOCK_DGRAM, 0)) < 0)  {
        perror ("socket");
        exit (-1);
      }

      strcpy (ifr.ifr_name, AuxBuff);
  
      if (ioctl (sd, SIOCGIFFLAGS, &ifr) == -1)  {
        perror ("ioctl(SIOCGIFFLAGS)");
        exit (-1);
      }

      if (Clear)  {
        ifr.ifr_flags &= ~IFF_PROMISC;
      }
      else  {
        ifr.ifr_flags |= IFF_PROMISC;
      }
 
      if (ioctl (sd, SIOCSIFFLAGS, &ifr) == -1)  {
        perror ("ioctl(SIOCSIFFLAGS)");
        exit (-1);
      }

      if (Clear)  {
        printf ("\tInterface [%s]  PROMISC OFF\n\n", AuxBuff);
      }
      else  {
        printf ("\tInterface [%s]  PROMISC ON\n\n", AuxBuff);
      }

      close (sd);
    }

    else  {
      printf ("\tSintax:  %s [[-]device]\n\n", argv[0]);
    }

    return 0;
}


